#include <glib.h>
#include <GL/gl.h>
#include "node.h"
#include "cube.h"

static void
cube_compute_extents (Node    *node,
		      Extents *extents)
{
    extents->x = 0.0;
    extents->y = 0.0;
    extents->z = 0.0;
    extents->width = 1.0;
    extents->height = 1.0;
    extents->depth = 1.0;
}

static void
cube_render (Node *node)
{
    CubeNode *cube = (CubeNode *)node;
    
    double wt = 1.0;
    double ht = 1.0;

#if 0
    glBegin (GL_QUADS);

    glColor3f (1.0, 0.0, 0.0);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (0.0f, 0.0f, 0.0f);
    glTexCoord2f (  wt, 0.0f); glVertex3f (1.0f, 0.0f, 0.0f);
    glTexCoord2f (  wt,   ht); glVertex3f (1.0f, 1.0f, 0.0f);
    glTexCoord2f (0.0f,   ht); glVertex3f (0.0f, 1.0f, 0.0f);

    glColor3f (0.0, 1.0, 0.0);
    glTexCoord2f (  wt, 0.0f); glVertex3f (0.0f, 0.0f, -1.0f);
    glTexCoord2f (  wt,   ht); glVertex3f (0.0f, 1.0f, -1.0f);
    glTexCoord2f (0.0f,   ht); glVertex3f (1.0f, 1.0f, -1.0f);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (1.0f, 0.0f, -1.0f);

    glColor3f (0.0, 0.0, 1.0);
    glTexCoord2f (0.0f,   ht); glVertex3f (0.0f, 1.0f, -1.0f);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (0.0f, 1.0f,  0.0f);
    glTexCoord2f (  wt, 0.0f); glVertex3f (1.0f, 1.0f,  0.0f);
    glTexCoord2f (  wt,   ht); glVertex3f (1.0f, 1.0f, -1.0f);

    glColor3f (1.0, 1.0, 0.0);
    glTexCoord2f (  wt,   ht); glVertex3f (0.0f, 0.0f, -1.0f);
    glTexCoord2f (0.0f,   ht); glVertex3f (1.0f, 0.0f, -1.0f);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (1.0f, 0.0f,  0.0f);
    glTexCoord2f (  wt, 0.0f); glVertex3f (0.0f, 0.0f,  0.0f);

    glColor3f (0.0, 1.0, 1.0);
    glTexCoord2f (  wt, 0.0f); glVertex3f (1.0f, 0.0f, -1.0f);
    glTexCoord2f (  wt,   ht); glVertex3f (1.0f, 1.0f, -1.0f);
    glTexCoord2f (0.0f,   ht); glVertex3f (1.0f, 1.0f,  0.0f);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (1.0f, 0.0f,  0.0f);

    glColor3f (1.0, 0.0, 1.0);
    glTexCoord2f (0.0f, 0.0f); glVertex3f (0.0f, 0.0f, -1.0f);
    glTexCoord2f (  wt, 0.0f); glVertex3f (0.0f, 0.0f,  0.0f);
    glTexCoord2f (  wt,   ht); glVertex3f (0.0f, 1.0f,  0.0f);
    glTexCoord2f (0.0f,   ht); glVertex3f (0.0f, 1.0f, -1.0f);

    glEnd ();
#endif

    glColor4f (1.0, 0.3, 0.7, 0.6);
    
    glPushMatrix ();

    if (cube->faces[0])
	cube->faces[0]->render (cube->faces[0]);

    glTranslatef (0.0, 1.0, 0.0);
    glRotatef (90, 1.0, 0.0, 0.0);

    if (cube->faces[1])
	cube->faces[1]->render (cube->faces[1]);

    glTranslatef (0.0, 1.0, 0.0);
    glRotatef (90, 1.0, 0.0, 0.0);

    if (cube->faces[2])
	cube->faces[2]->render (cube->faces[2]);

    glTranslatef (0.0, 1.0, 0.0);
    glRotatef (90, 1.0, 0.0, 0.0);
    
    if (cube->faces[3])
	cube->faces[3]->render (cube->faces[3]);

    /* back to normal */
    glTranslatef (0.0, 1.0, 0.0);
    glRotatef (90, 1.0, 0.0, 0.0);

    glTranslatef (1.0, 0.0, 0.0);
    glRotatef (-90, 0.0, 1.0, 0.0);

    if (cube->faces[4])
	cube->faces[4]->render (cube->faces[4]);

    glTranslatef (0.0, 0.0, 1.0);

    if (cube->faces[5])
	cube->faces[5]->render (cube->faces[5]);
    
    glPopMatrix ();
    
}

CubeNode *
cube_node_new (void)
{
    CubeNode *node = g_new0 (CubeNode, 1);

    node->node.render = cube_render;
    node->node.compute_extents = cube_compute_extents;
    
    return node;
}

void
cube_set_face (CubeNode *cube,
	       int face_no,
	       Node *child)
{
    g_return_if_fail (face_no < 6);
    
    cube->faces[face_no] = child;

    node_queue_repaint ((Node *)cube);
}
