#include "wsint.h"

#include <X11/extensions/XShm.h>

#include <sys/ipc.h>
#include <sys/shm.h>

#if 0
static void
fill_junk (guchar *mem, gsize size)
{
    int i;
    
    for (i = 0; i < size; ++i)
    {
	mem[i] = g_random_int();
    }
}
#endif

static gint
format_get_depth (WsFormat format)
{
    switch (format)
    {
    case WS_FORMAT_UNKNOWN:
	return -1;
    case WS_FORMAT_RGB_16:
	return 16;
    case WS_FORMAT_RGB_24:
	return 24;
    case WS_FORMAT_RGB_32:
	return 32;
    case WS_FORMAT_ARGB_32:
	return 32;
    default:
	return -1;
    }
}

Visual *
format_to_xvisual (WsFormat format, Screen *screen)
{
    int i;
    
    for (i = 0; i < screen->ndepths; ++i)
    {
	Depth *depth_str = &(screen->depths[i]);
	int j;
	
	for (j = 0; j < depth_str->nvisuals; ++j)
	{
	    Visual *visual = &(depth_str->visuals[j]);
	    int depth = depth_str->depth;
	    
	    switch (format)
	    {
	    case WS_FORMAT_RGB_16:
		if (depth == 16 &&
		    visual->red_mask == 0xf800 &&
		    visual->green_mask == 0x7e0 &&
		    visual->blue_mask == 0x1f)
		{
		    return visual;
		}
		break;
		
	    case WS_FORMAT_RGB_24:
		if (depth == 24 &&
		    visual->red_mask == 0xff0000 &&
		    visual->green_mask == 0xff00 &&
		    visual->blue_mask == 0xff)
		{
		    return visual;
		}
		break;

	    case WS_FORMAT_RGB_32:
	    case WS_FORMAT_ARGB_32:
		if (depth == 32 &&
		    visual->red_mask == 0xff0000 &&
		    visual->green_mask == 0xff00 &&
		    visual->blue_mask == 0xff)
		{
		    return visual;
		}
		break;
		
	    case WS_FORMAT_UNKNOWN:
		break;
	    }
	}
	
    }

    return NULL;
}

guint
ws_shm_pixmap_get_bytes_per_row (WsShmPixmap *pixmap)
{
    /* FIXME */

#if 0
    g_print ("width: %d   bpr: %d\n", pixmap->width, pixmap->bytes_per_row);
#endif
    
    return pixmap->bytes_per_row;
}

WsShmPixmap *
ws_shm_pixmap_new (WsDrawable  *drawable,
		   int          width,
		   int          height,
		   WsFormat     format)
{
    WsShmPixmap *pixmap;
    Display *xdisplay;
    Ws *ws = drawable->ws;
    Visual *visual;
    XImage *image;
    XID xid;
    
    g_return_val_if_fail (drawable != NULL, NULL);
    g_return_val_if_fail (width > 0, NULL);
    g_return_val_if_fail (height > 0, NULL);
    g_return_val_if_fail (format != WS_FORMAT_UNKNOWN, NULL);

    pixmap = g_new0 (WsShmPixmap, 1);
    
    visual = format_to_xvisual (format, ws_drawable_query_screen (drawable)->xscreen);

    xdisplay = ws->xdisplay;
    
    image = XCreateImage (xdisplay,
			  visual,
			  format_get_depth (format),
			  ZPixmap,
			  0,
			  0,
			  width,
			  height,
			  32,
			  0);
    
    pixmap->bytes_per_row = image->bytes_per_line;

    xdisplay = drawable->ws->xdisplay;
    
    pixmap->shm_info.shmid =
	shmget (IPC_PRIVATE, image->bytes_per_line * height,
		IPC_CREAT|0600);
    
    if (pixmap->shm_info.shmid < 0)
    {
	g_print ("asdf\n");
	return NULL;
    }
    
    pixmap->shm_info.shmaddr = (char *)shmat (pixmap->shm_info.shmid, 0, 0);
    
    if (pixmap->shm_info.shmaddr == ((char *)-1))
    {
	shmctl (pixmap->shm_info.shmid, IPC_RMID, 0);
	return NULL;
    }
    
    pixmap->shm_info.readOnly = FALSE;
    
    /* FIXME: we should check for errors around this */
    
    XShmAttach (xdisplay, &pixmap->shm_info);
    XSync (xdisplay, False);
    
    /* Detach now so we clean up on abnormal exit */
    shmctl (pixmap->shm_info.shmid, IPC_RMID, 0);

    xid = XShmCreatePixmap (ws->xdisplay,
			       drawable->xid,
			       pixmap->shm_info.shmaddr,
			       &pixmap->shm_info,
			       width, height, format_get_depth (format));
	       
    _ws_drawable_init (&(pixmap->pixmap.drawable), ws, xid);
    
    pixmap->format = WS_FORMAT_RGB_32;
    pixmap->width = width;
    pixmap->height = height;
    
    return pixmap;
}

const guchar *
ws_shm_pixmap_get_data (WsShmPixmap *pixmap)
{
    return (guchar *)pixmap->shm_info.shmaddr;
}

void
ws_shm_pixmap_unref (WsShmPixmap *pixmap)
{
    if (--pixmap->pixmap.drawable.ref_count == 0)
    {
	XFreePixmap (pixmap->pixmap.drawable.ws->xdisplay,
		     pixmap->pixmap.drawable.xid);
    }
}

WsFormat
ws_shm_pixmap_get_format (WsShmPixmap *pixmap)
{
    return pixmap->format;
}
